import { Injectable } from '@angular/core';
import { Observable, of, delay } from 'rxjs';
import { ProductModel } from '../interface/product.interface';
import { MOCK_PRODUCTS } from '../const/products.const';

/**
 * ProductService - Quản lý dữ liệu sản phẩm
 * Demo về cách tạo service với @Injectable
 */
@Injectable({
  providedIn: 'root'
})
export class ProductService {
  
  constructor() {
    console.log('🛍️ ProductService instance được tạo');
  }

  /**
   * Lấy danh sách tất cả sản phẩm
   * Simulate API call với delay
   */
  getAllProducts(): Observable<ProductModel[]> {
    console.log('📦 Loading products...');
    return of(MOCK_PRODUCTS).pipe(
      delay(500) // Simulate network delay
    );
  }

  /**
   * Lấy sản phẩm theo ID
   */
  getProductById(id: number): Observable<ProductModel | undefined> {
    const product = MOCK_PRODUCTS.find(p => p.id === id);
    return of(product).pipe(delay(200));
  }

  /**
   * Tìm kiếm sản phẩm theo tên
   */
  searchProducts(query: string): Observable<ProductModel[]> {
    const filteredProducts = MOCK_PRODUCTS.filter(product =>
      product.name.toLowerCase().includes(query.toLowerCase()) ||
      product.description?.toLowerCase().includes(query.toLowerCase())
    );
    return of(filteredProducts).pipe(delay(300));
  }

  /**
   * Lấy sản phẩm theo category (mock)
   */
  getProductsByCategory(category: string): Observable<ProductModel[]> {
    // Mock implementation - trong thực tế sẽ filter theo category
    return of(MOCK_PRODUCTS).pipe(delay(400));
  }
}
